@@ -2,7 +2,7 @@ import logging
 import os
 import tempfile
 
-from . import activationsfunc, compat, hyper, mat_gen, nodes, observables, types
+from . import activationsfunc, compat, hyper, mat_gen, nodes, observables, type
 from ._version import __version__
 from .compat import load_compat
 from .compat.utils.save import load
@@ -15,7 +15,7 @@ from .utils.random import set_seed
 logger = logging.getLogger(__name__)
 
 _TEMPDIR = os.path.join(tempfile.gettempdir(), "reservoirpy-temp")
-if not os.path.exists(_TEMPDIR):
+if not os.path.exists(_TEMPDIR):  # pragma: no cover
     try:
         os.mkdir(_TEMPDIR)
     except OSError:
