@@ -1,13 +1,14 @@
 # Author: Nathan Trouvain at 15/02/2022 <nathan.trouvain@inria.fr>
 # Licence: MIT License
 # Copyright: Xavier Hinaut (2018) <xavier.hinaut@inria.fr>
-from contextlib import contextmanager
-from typing import Union, Dict, Sequence, Any, Iterator
 from abc import ABC, abstractmethod
+from contextlib import contextmanager
+from typing import Any, Dict, Iterator, Sequence, Union
 from uuid import uuid4
 
 import numpy as np
-from .type import Shape, MappedData
+
+from .type import MappedData, Shape
 from .utils import progress
 from .utils.validation import check_vector, is_mapping
 
